@@ -404,15 +404,32 @@ class histogram:
 
     :param bin_edges: The bin-edges.
     :param right: Whether the bin includes the right edge (or the left edge) see numpy.digitize.
+    :param bound_error: What to do if a sample falls out of the bin range:
+
+        - ``"raise"``: raise an error
+        - ``"ignore"``: ignore the data that are out of range
+        - ``"norm"``: change the normalisation of the density
+
     :param count: The initial count (default: zeros).
     """
 
-    def __init__(self, bin_edges: ArrayLike, right: bool = False, count: ArrayLike = None):
+    def __init__(
+        self,
+        bin_edges: ArrayLike,
+        right: bool = False,
+        bound_error: str = "raise",
+        count: ArrayLike = None,
+    ):
 
         assert np.all(np.diff(bin_edges) > 0) or np.all(np.diff(bin_edges) < 0)
 
         self.right = right
         self.bin_edges = np.array(bin_edges).astype(np.float64)
+        self.bound_error = bound_error
+        self.count_left = 0
+        self.count_right = 0
+        self.bound_left = bin_edges[0]
+        self.bound_right = bin_edges[-1]
 
         if count is not None:
             assert len(count) == len(bin_edges) - 1
@@ -430,6 +447,7 @@ class histogram:
         min_width: float = None,
         integer: bool = False,
         bin_edges: ArrayLike = None,
+        bound_error: str = "raise",
     ):
         r"""
         Construct a histogram from data.
@@ -453,6 +471,12 @@ class histogram:
 
         :param bin_edges: Specify the bin-edges (overrides ``bins`` and ``mode``).
 
+        :param bound_error: What to do if a sample falls out of the bin range:
+
+            - ``"raise"``: raise an error
+            - ``"ignore"``: ignore the data that are out of range
+            - ``"norm"``: change the normalisation of the density
+
         :return: The :py:class:`Histogram` object.
         """
 
@@ -471,9 +495,10 @@ class histogram:
         if integer:
             bin_edges = detail.histogram_bin_edges_integer(bin_edges)
 
-        return cls(
-            bin_edges, right=True, count=np.histogram(data, bins=bin_edges, density=False)[0]
-        )
+        bin_edges = np.array(bin_edges)
+        bin_edges[0] -= np.finfo(bin_edges.dtype).eps
+        bin_edges[-1] += np.finfo(bin_edges.dtype).eps
+        return cls(bin_edges, right=True, bound_error=bound_error).add_sample(data)
 
     def strip(self, min_count: int = 0):
         """
@@ -484,6 +509,7 @@ class histogram:
 
         self.lstrip(min_count)
         self.rstrip(min_count)
+        return self
 
     def lstrip(self, min_count: int = 0):
         """
@@ -495,6 +521,7 @@ class histogram:
         i = np.argmax(self.count > min_count)
         self.count = self.count[i:]
         self.bin_edges = self.bin_edges[i:]
+        return self
 
     def rstrip(self, min_count: int = 0):
         """
@@ -506,6 +533,7 @@ class histogram:
         i = len(self.count) - np.argmax(self.count[::-1] > min_count)
         self.count = self.count[:i]
         self.bin_edges = self.bin_edges[: i + 1]
+        return self
 
     def interp(self, bin_edges: ArrayLike):
         """
@@ -519,6 +547,7 @@ class histogram:
         m = 0.5 * (bin_edges[:-1] + bin_edges[1:])
         self.count = np.interp(m, self.x, self.count)
         self.bin_edges = bin_edges
+        return self
 
     def squash(self, n: int):
         """
@@ -531,7 +560,7 @@ class histogram:
         if n >= self.count.size:
             self.count = np.sum(self.count).reshape(1)
             self.bin_edges = self.bin_edges[[0, -1]]
-            return
+            return self
 
         m = self.count.size // n
         s = m * n
@@ -542,6 +571,8 @@ class histogram:
         else:
             self.bin_edges = self.bin_edges[::n]
 
+        return self
+
     def merge_right(self, index: ArrayLike):
         """
         Merge the bins to the right of ``index`` into ``index``.
@@ -549,6 +580,9 @@ class histogram:
         :param index: The indices of the bin to merge into.
         """
 
+        if isinstance(index, int):
+            index = [index]
+
         index = np.array(index)
         index = np.sort(np.where(index < 0, index + self.count.size, index))
 
@@ -558,6 +592,7 @@ class histogram:
         self.count[index] += self.count[index + 1]
         self.count = np.delete(self.count, index + 1)
         self.bin_edges = np.delete(self.bin_edges, index + 1)
+        return self
 
     def merge_left(self, index: ArrayLike):
         """
@@ -566,6 +601,9 @@ class histogram:
         :param index: The indices of the bin to merge into.
         """
 
+        if isinstance(index, int):
+            index = [index]
+
         index = np.array(index)
         index = np.sort(np.where(index < 0, index + self.count.size, index))
 
@@ -575,6 +613,7 @@ class histogram:
         self.count[index - 1] += self.count[index]
         self.count = np.delete(self.count, index)
         self.bin_edges = np.delete(self.bin_edges, index)
+        return self
 
     def as_integer(self):
         """
@@ -601,7 +640,26 @@ class histogram:
         """
 
         bin = np.digitize(data, self.bin_edges, self.right) - 1
+
+        left = bin < 0
+        right = bin >= self.count.size
+        nleft = np.sum(left)
+        nright = np.sum(right)
+        self.count_left += nleft
+        self.count_right += nright
+
+        if nleft:
+            self.bound_left = min([self.bound_left, np.min(data[left])])
+
+        if nright:
+            self.bound_right = max([self.bound_right, np.max(data[right])])
+
+        if self.bound_error == "raise" and (nleft or nright):
+            raise ValueError("Data out of bin range")
+
+        bin = bin[np.logical_and(~left, ~right)]
         self.count += np.bincount(bin, minlength=self.count.size).astype(np.uint64)
+        return self
 
     @property
     def x(self) -> ArrayLike:
@@ -618,7 +676,28 @@ class histogram:
         """
 
         count = self.count.astype(np.float64)
-        return count / np.sum(np.diff(self.bin_edges) * count)
+        norm = np.sum(count)
+
+        if self.bound_error == "norm":
+            norm += self.count_left + self.count_right
+
+        return count / (np.diff(self.bin_edges) * norm)
+
+    @property
+    def p(self) -> ArrayLike:
+        """
+        The probability density function at the bin.
+        """
+
+        return self.density
+
+    @property
+    def plot(self) -> tuple[ArrayLike, ArrayLike]:
+        """
+        Alias for ``(x, density)``.
+        """
+
+        return (self.x, self.density)
 
 
 if __name__ == "__main__":
