@@ -14,12 +14,57 @@ class Test_norm(unittest.TestCase):
 
         data = [0, 0, 0, 1, 1, 2]
         bin_edges = [-0.5, 0.5, 1.5, 2.5]
+        c, _ = np.histogram(data, bins=bin_edges, density=False)
         p, _ = np.histogram(data, bins=bin_edges, density=True)
 
         hist = enstat.histogram.from_data(data, bin_edges=bin_edges)
 
+        self.assertTrue(np.allclose(hist.count, c))
         self.assertTrue(np.allclose(hist.density, p))
 
+    def test_density_random(self):
+
+        data = np.random.random([1001])
+
+        for bin_edges in [
+            np.logspace(-5, 1, 1000),
+            np.linspace(0, 1, 1000),
+            np.sort(np.random.random([10])),
+        ]:
+
+            c, _ = np.histogram(data, bins=bin_edges, density=False)
+            p, _ = np.histogram(data, bins=bin_edges, density=True)
+            hist = enstat.histogram.from_data(data, bin_edges=bin_edges, bound_error="ignore")
+            self.assertTrue(np.allclose(hist.count, c))
+            self.assertTrue(np.allclose(hist.density, p))
+
+    def test_density_random_binning(self):
+
+        data = np.random.random([1001])
+
+        for mode in ["equal", "log", "uniform", "voronoi"]:
+
+            hist = enstat.histogram.from_data(data, bins=50, mode=mode)
+            c, _ = np.histogram(data, bins=hist.bin_edges, density=False)
+            p, _ = np.histogram(data, bins=hist.bin_edges, density=True)
+            self.assertTrue(np.all(hist.count == c))
+            self.assertTrue(np.allclose(hist.density, p))
+
+    def test_bound_error_norm(self):
+
+        data = np.random.random([1001])
+
+        r = np.random.random([30])
+        bin_edges = np.min(data) + np.cumsum(0.1 * r)
+        bin_edges = bin_edges[bin_edges < np.max(data)]
+        bin_edges_full = np.hstack((np.min(data), bin_edges, np.max(data)))
+
+        hist = enstat.histogram.from_data(data, bin_edges=bin_edges, bound_error="norm")
+        hist_full = enstat.histogram.from_data(data, bin_edges=bin_edges_full, bound_error="raise")
+
+        self.assertTrue(np.all(hist.count == hist_full.count[1:-1]))
+        self.assertTrue(np.allclose(hist.p, hist_full.p[1:-1]))
+
     def test_mid(self):
 
         bin_edges = [-0.5, 0.5, 1.5, 2.5]
@@ -39,6 +84,8 @@ class Test_accumulate(unittest.TestCase):
 
         data = np.random.random([100, 100])
         bin_edges = np.linspace(0, 1, 21)
+
+        c, _ = np.histogram(data.ravel(), bins=bin_edges, density=False)
         p, _ = np.histogram(data.ravel(), bins=bin_edges, density=True)
 
         hist = enstat.histogram(bin_edges=bin_edges)
@@ -46,8 +93,35 @@ class Test_accumulate(unittest.TestCase):
         for i in range(data.shape[0]):
             hist += data[i, :]
 
+        self.assertEqual(hist.count_left, 0)
+        self.assertEqual(hist.count_right, 0)
+
+        self.assertTrue(np.all(hist.count == c))
         self.assertTrue(np.allclose(hist.density, p))
 
+    def test_accumulate_bin_edges(self):
+
+        data = np.random.random([100, 100])
+
+        for mode in ["equal", "log", "uniform", "voronoi"]:
+
+            hist = enstat.histogram.from_data(data.ravel(), bins=51, mode=mode)
+            bin_edges = np.copy(hist.bin_edges)
+
+            c, _ = np.histogram(data.ravel(), bins=bin_edges, density=False)
+            p, _ = np.histogram(data.ravel(), bins=bin_edges, density=True)
+
+            hist = enstat.histogram(bin_edges=bin_edges)
+
+            for i in range(data.shape[0]):
+                hist += data[i, :]
+
+            self.assertEqual(hist.count_left, 0)
+            self.assertEqual(hist.count_right, 0)
+
+            self.assertTrue(np.all(hist.count == c))
+            self.assertTrue(np.allclose(hist.density, p))
+
 
 class Test_strip(unittest.TestCase):
     """
@@ -209,6 +283,9 @@ class Test_squash(unittest.TestCase):
         hist = enstat.histogram(bin_edges=bin_edges, count=count)
         hist.squash(2)
 
+        self.assertEqual(0, hist.count_left)
+        self.assertEqual(0, hist.count_right)
+
         self.assertEqual([3, 3, 3], hist.count.tolist())
         self.assertEqual([0, 2, 4, 6], hist.bin_edges.tolist())
 
@@ -220,6 +297,9 @@ class Test_squash(unittest.TestCase):
         hist = enstat.histogram(bin_edges=bin_edges, count=count)
         hist.squash(3)
 
+        self.assertEqual(0, hist.count_left)
+        self.assertEqual(0, hist.count_right)
+
         self.assertEqual([4, 5], hist.count.tolist())
         self.assertEqual([0, 3, 6], hist.bin_edges.tolist())
 
@@ -288,6 +368,9 @@ class Test_histogram_bin_edges_voronoi(unittest.TestCase):
         data = np.array([0, 0, 1, 1, 1, 2, 2, 3, 4, 5, 6])
         hist = enstat.histogram.from_data(data, bins=10, mode="voronoi")
 
+        self.assertEqual(hist.count_left, 0)
+        self.assertEqual(hist.count_right, 0)
+
         self.assertTrue(
             np.allclose(hist.bin_edges, np.array([-0.5, 0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 6.5]))
         )
@@ -297,6 +380,9 @@ class Test_histogram_bin_edges_voronoi(unittest.TestCase):
         data = np.array([8, 8, 1, 1, 1, 2, 2, 3, 4, 5, 6])
         hist = enstat.histogram.from_data(data, bins=10, mode="voronoi")
 
+        self.assertEqual(hist.count_left, 0)
+        self.assertEqual(hist.count_right, 0)
+
         self.assertTrue(np.allclose(hist.bin_edges, np.array([0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 7, 9])))
 
 
